/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#ifndef OAPR_IO_FILE_HPP
#define OAPR_IO_FILE_HPP
#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_pool.hpp"
#include "oapr/oapr_io.hpp"
#include "apr_file_io.h"

FILE_VERSION("$Revision: 1.8 $",oapr_io_file_hpp);

namespace oapr
{
  class LIBOAPR_API basic_file : public basic_io
  {
  public:
    basic_file(const pool::pool_ptr& p)
      : basic_io(p)
	,m_file(0)
	,m_isOpen(false)
    {
    }
    basic_file(const basic_file& f)
      : basic_io(f),
	m_fileName(f.m_fileName),
	m_file(f.m_file),
	m_isOpen(f.m_isOpen)
    {
    }
    virtual ~basic_file()
    {
    }
    virtual std::string name()const{return m_fileName;}
    virtual apr_status_t open(const std::string& addr,
		      apr_int32_t flags,
		      apr_int32_t perms = APR_OS_DEFAULT);
    /**\todo operations we need
     * - duplicate
     * - setaside ?
     * - seek
     * - lock/unlock ?
     */

    /* generic buffer operations from the io class */
    virtual apr_status_t close();
    virtual bool eof();
    virtual bool is_open()const{return m_isOpen;}

    virtual apr_size_t write(const void* buff,
			     apr_size_t toWrite,
			     bool full=false);
    virtual apr_size_t read( void* buff,
			     apr_size_t toRead,
			     bool full=false)const;

    virtual apr_status_t flush();
  protected:
    std::string m_fileName;
    apr_file_t *m_file;
    bool m_isOpen;
  };
  LIBOAPR_API apr_status_t remove(const basic_file& f);
  LIBOAPR_API apr_status_t remove(const std::string &nm);
  LIBOAPR_API apr_status_t rename(const basic_file& f,const std::string& to);
  LIBOAPR_API apr_status_t rename(const std::string& from,const std::string& to);
  LIBOAPR_API apr_status_t copy(const basic_file& f,const std::string& to,
		    apr_fileperms_t perms=APR_FILE_SOURCE_PERMS);
  LIBOAPR_API apr_status_t  copy(const std::string& from,const std::string& to,
		    apr_fileperms_t perms=APR_FILE_SOURCE_PERMS);
  LIBOAPR_API apr_status_t append(const basic_file& f,const std::string& to,
		      apr_fileperms_t perms=APR_FILE_SOURCE_PERMS);
  LIBOAPR_API apr_status_t append(const std::string& f,const std::string& to,
		      apr_fileperms_t perms=APR_FILE_SOURCE_PERMS);

};
#endif
